import datetime
import json

from django.db import models

NULLABLE = {'blank': True, 'null': True}


class Group(models.Model):
    t_id = models.CharField(max_length=100, default='', verbose_name='Telegram ID')
    username = models.CharField(max_length=200, default='', verbose_name='Username', **NULLABLE)
    name = models.CharField(max_length=200, default='', verbose_name='Имя', **NULLABLE)

    class Meta:
        verbose_name = 'группа'
        verbose_name_plural = 'группы'


class BaseBot(models.Model):
    t_id = models.CharField(max_length=100, default='', verbose_name='Telegram ID')
    token = models.CharField(max_length=255, **NULLABLE, verbose_name='Bot token')
    name = models.CharField(max_length=250, default='', verbose_name='Name')
    username = models.CharField(max_length=250, default='', verbose_name='Username')
    created = models.DateTimeField(auto_now_add=True, verbose_name='Creation date')

    class Meta:
        abstract = True

    def __str__(self):
        return self.t_id


class Bot(BaseBot):
    pass


class Message(models.Model):
    message_id = models.BigIntegerField()
    update_id = models.BigIntegerField()
    from_id = models.BigIntegerField()
    text = models.TextField(**NULLABLE)
    data = models.TextField(**NULLABLE)

    created = models.DateTimeField(auto_now_add=True)


class BaseUser(models.Model):
    t_id = models.CharField(max_length=100, default='', verbose_name='Telegram ID')
    username = models.CharField(max_length=200, default='', verbose_name='Username', null=True, blank=True)
    name = models.CharField(max_length=200, default='', verbose_name='Имя', null=True, blank=True)
    last_message_id = models.BigIntegerField(**NULLABLE, verbose_name='Номер последнего сообщения')
    want_action = models.CharField(max_length=250, **NULLABLE)
    last_path = models.CharField(max_length=250, **NULLABLE, default='/start')
    path = models.CharField(max_length=250, **NULLABLE)
    state_information = models.TextField(**NULLABLE)
    created = models.DateTimeField('Creation date', default=datetime.datetime.now)

    groups = models.ManyToManyField('Group', blank=True)

    def update_last_sent_message(self, response_content):
        response_dict = json.loads(response_content)
        if response_dict.get('ok'):
            result = response_dict.get('result')
            if isinstance(result, dict):
                last_message_id = result.get('message_id')
                self.last_message_id = last_message_id
            else:
                self.last_message_id = None
            self.save()

    def update_path(self, path):
        """Update path."""
        self.last_path = self.path if self.path else '/start'
        self.path = path
        self.save()

    def clean_state(self):
        """ Clean state of user, can be used in actions """
        self.want_action = None
        self.state_information = None
        self.save()

    class Meta:
        abstract = True

    def __str__(self):
        return f'{self.t_id}'


class User(BaseUser):
    pass


class Constructor(models.Model):
    pass



# Generated by Django 4.2.6 on 2023-10-10 14:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0003_user_last_message_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='want_action',
            field=models.CharField(blank=True, max_length=250, null=True),
        ),
    ]



from oscarbot.menu import Menu, Button
from oscarbot.messages import get_msg
from oscarbot.response import TGResponse


def start(_):
    return TGResponse(
        need_update=False
    )


def first_question(_):
    menu = Menu([
        Button("Да", callback="/diagnostic/"),
    ])
    return TGResponse(
        message=get_msg(text_args=['qweqwe', 'qwdqweqw']),
        menu=menu,
    )



import inspect
import os

import yaml
from django.conf import settings
from typing_extensions import LiteralString

from oscarbot.router import Router


class ResponseMessage:
    FILE_NAME = 'messages.yaml'

    @classmethod
    def find_message(cls, alias, path='messages'):
        for app_item in settings.OSCARBOT_APPS:
            app_message_file_path = os.path.join(settings.BASE_DIR, app_item, cls.FILE_NAME)
            with open(app_message_file_path, 'r', encoding='utf8') as messages_file:
                messages_data = yaml.load(messages_file, yaml.Loader)
                message_text = messages_data[path].get(alias)
                if message_text:
                    return message_text

        return None

    @classmethod
    def collect(cls):
        """ Collect all routers in messages if not exists """
        for app_item in settings.OSCARBOT_APPS:

            new_keys = []
            router = Router('')
            for route in router.routes:
                new_keys.append(route.func.__name__)

            app_message_file_path = os.path.join(settings.BASE_DIR, app_item, cls.FILE_NAME)

            with open(app_message_file_path, 'r', encoding='utf8') as messages_file:
                messages_data = yaml.load(messages_file, yaml.Loader)

            for key in new_keys:

                if key not in messages_data.get('messages', {}).keys():
                    messages_data['messages'][key] = '\'  \''

            cls._save_to_file(messages_data, app_message_file_path)

    @staticmethod
    def _save_to_file(data: dict, path: LiteralString | str | bytes) -> None:
        """ Update file with messages """
        with open(path, 'w', encoding='utf8') as messages_file:
            messages_file.write("# To set multilines message text use triple quotes ''' set text here '''\n\n")
            yaml.preserve_quotes = True
            yaml.explicit_start = True
            yaml.dump(data=data, stream=messages_file)

    @classmethod
    def make_template(cls, force=False):
        """ Create empty default messages files """
        for app_item in settings.OSCARBOT_APPS:
            app_message_file_path = os.path.join(settings.BASE_DIR, app_item, cls.FILE_NAME)

            if os.path.exists(app_message_file_path) and not force:
                print(f'Messages file for "{app_item}" application exists')
            else:
                data = {
                    'messages': {'start': '\'  \''},
                    'defaults': {'text_processor': '\'  \'', 'dont_know': '\'Sorry, i do not know this command\'',
                                 'voice_processor': '\'  \''}
                }

                cls._save_to_file(data, app_message_file_path)


def get_msg(alias=None, text_args: list | None = None) -> str:
    """ Find message by alias """
    if text_args is None:
        text_args = []

    if alias is None:
        alias = inspect.stack()[1][3]

    final_message = ResponseMessage.find_message(alias)
    if final_message:
        counter = 1
        for arg in text_args:
            final_message = final_message.replace(f'{{{counter}}}', arg)
            counter += 1

        return final_message

    return ResponseMessage.find_message('dont_know', 'defaults')



import json

import requests
from django.conf import settings
from django.core.management import BaseCommand

from oscarbot.bot_logger import log
from oscarbot.services import get_bot_model
from oscarbot.views import handle_content


class Command(BaseCommand):
    """Command"""

    class BotData:
        """Bot Data"""
        token = None

    def handle(self, *args, **options):
        bot_model = get_bot_model()
        bot = bot_model.objects.all().first()
        if not bot:
            bot = self.BotData()
            bot.token = settings.TELEGRAM_API_TOKEN if getattr(settings, 'TELEGRAM_API_TOKEN', None) else None
        offset = 0
        try:
            while True:
                url = f'{settings.TELEGRAM_URL}{bot.token}/getUpdates?offset={offset}'
                response = requests.get(url, timeout=60)
                total_message = b''
                for message in response:
                    total_message += message
                body = total_message.decode('utf-8')
                body = body.replace('\n', '')
                content = json.loads(body)
                if content.get('ok'):
                    if len(content['result']) > 0:
                        offset = int(content['result'][0]['update_id']) + 1
                        if offset > 0:
                            handle_content(bot.token, content['result'][0])
                else:
                    raise ValueError
        except ValueError as e:
            log.error(f'Token from Telegram not found\n{e}')
        except AttributeError as e:
            log.error(f'Add the bot token to the database in {bot_model}\n'
                      f'Or settings.py attribute TELEGRAM_API_TOKEN\n{e}')
        except KeyboardInterrupt:
            log.info(f'Exit bot server')



# Generated by Django 4.2.6 on 2023-10-09 18:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message_id', models.BigIntegerField()),
                ('update_id', models.BigIntegerField()),
                ('from_id', models.BigIntegerField()),
                ('text', models.TextField(blank=True, null=True)),
                ('data', models.TextField(blank=True, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('t_id', models.CharField(default='', max_length=100, verbose_name='Telegram ID')),
                ('username', models.CharField(default='', max_length=200, verbose_name='Username')),
                ('name', models.CharField(default='', max_length=200, verbose_name='Имя')),
            ],
        ),
    ]



from oscarbot.menu import Menu
from oscarbot.tests.base import TGTestCase


class TestExample(TGTestCase):
    """Example test case."""

    def test_example(self):
        """Example test method."""
        example_router = '/start'

        response = self.call_router(example_router)
        self.assertMessage(response, 'Привет! Мы здесь, чтобы продиагностировать твой бизнес. Начнем?')
        self.assertMenu(response, Menu([]))
        self.assertNeedUpdate(response, True)



# Generated by Django 4.2.6 on 2024-01-14 13:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0004_user_want_action'),
    ]

    operations = [
        migrations.CreateModel(
            name='Constructor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
    ]



from example.views import start, first_question
from oscarbot.router import route

routes = [
    route('/start', start),
    route('/diagnostic/', first_question),
    route('/hello/', first_question),
    route('/hello/<pk>/', first_question),
]



# Generated by Django 4.2.6 on 2023-10-10 10:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oscarbot', '0002_message_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='last_message_id',
            field=models.BigIntegerField(blank=True, null=True, verbose_name='Номер последнего сообщения'),
        ),
    ]



import json
import inspect
from django.conf import settings

from oscarbot.bot import Bot
from oscarbot.bot_logger import log
from oscarbot.messages import get_msg


class TGResponseBase:
    def __init__(
            self,
            menu=None,
            need_update: bool | None = None,
            is_delete_message=False,
            has_spoiler=False,
            protect=False,
            disable_web_page_preview=False,
    ):
        self.menu = menu

        need_update_setting = settings.TELEGRAM_NEED_UPDATE if getattr(settings, 'TELEGRAM_NEED_UPDATE', None) else True
        self.need_update = need_update if need_update is not None else need_update_setting

        self.has_spoiler = has_spoiler
        self.protect = protect

        self.is_delete_message = is_delete_message

        self.disable_web_page_preview = disable_web_page_preview

        self.tg_bot = None
        self.parse_mode = settings.TELEGRAM_PARSE_MODE if getattr(settings, 'TELEGRAM_PARSE_MODE', None) else 'HTML'


class TGResponseMedia:

    def __init__(
            self,
            photo=None,
            attache=None,
            video=None,
            file=None,
            media_group: list[dict] = None,
            media_group_type='photo',
            **kwargs
    ):
        super().__init__(**kwargs)
        self.attache = attache
        self.photo = photo
        self.video = video
        self.file = file
        self.media_group = media_group if media_group else None
        self.media_group_type = media_group_type


class TGResponse(TGResponseMedia, TGResponseBase):

    def __init__(
            self,
            message: str | None = None,
            callback_text='',
            callback_url=False,
            show_alert=False,
            cache_time=None,
            **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.callback_url = callback_url
        self.callback_text = callback_text
        self.show_alert = show_alert
        self.cache_time = cache_time

        if message is None:
            self.message = get_msg(inspect.stack()[1][3])
        elif message.startswith('#'):
            self.message = get_msg(message.replace('#', ''))
        else:
            self.message = message

    def __collect_data_to_send(self, user, group, t_id=None) -> dict:
        t_id_for_response = t_id
        if group:
            t_id_for_response = group.t_id
        elif user is not None:
            t_id_for_response = user.t_id
        data_to_send = {
            'chat_id': t_id_for_response,
            'message': self.message,
            'reply_keyboard': self.menu,
            'photo': self.photo,
            'video': self.video,
            'protect_content': self.protect,
            'parse_mode': self.parse_mode,
            'disable_web_page_preview': self.disable_web_page_preview,
            'file': self.file,
            'is_delete_message': self.is_delete_message,
        }

        if self.media_group:
            data_to_send['media_group'] = self.media_group
            data_to_send['media_group_type'] = self.media_group_type
            data_to_send['has_spoiler'] = self.has_spoiler
            self.need_update = False

        if self.photo:
            self.need_update = False

        return data_to_send

    @staticmethod
    def __get_message_id(user, content):
        update_chat_message = True
        update_chat_attr = getattr(settings, 'UPDATE_CHAT_MESSAGE', None)
        if update_chat_attr is not None:
            update_chat_message = update_chat_attr

        if update_chat_message:
            message_id = None
            if content:
                message = content.get('message')
                if not message:
                    callback_query = content.get('callback_query')
                    message = callback_query.get('message') if callback_query else None
                if message:
                    message_id = message.get('message_id')
        else:
            message_id = user.last_message_id

        return message_id

    def send(self, token, user=None, group=None, content=None, t_id=None):
        self.tg_bot = Bot(token)
        if content and (self.callback_text or self.callback_url):
            self.send_callback(content)
        if self.menu:
            self.menu = self.menu.build()


        data_to_send = self.__collect_data_to_send(user, group, t_id)

        message_id = self.__get_message_id(user, content)
        data_to_send['message_delete'] = message_id

        if self.need_update:
            response_content = self.tg_bot.update_message(**data_to_send, message_id=message_id)
            response_dict = json.loads(response_content)
            if not response_dict.get('ok') and self.is_delete_message:
                response_content = self.tg_bot.update_message(**data_to_send, message_id=user.last_message_id)
                response_dict = json.loads(response_content)
            check_update = response_dict.get('ok')
            if isinstance(check_update, bool) and not check_update:
                description = response_dict.get('description')
                message_errors = [
                    'Bad Request: there is no text in the message to edit',
                ]
                if description and description in message_errors:
                    response_content = self.tg_bot.send_message(**data_to_send)
        else:
            response_content = self.tg_bot.send_message(**data_to_send)
        log.info(response_content)
        if user:
            user.update_last_sent_message(response_content)

    def can_send(self):
        if self.message is not None or self.video is not None or self.photo is not None:
            return True
        return False

    def send_callback(self, content):
        """Send callback"""
        callback_query = content.get('callback_query') if content else None
        callback_query_id = callback_query.get('id') if callback_query else None
        if callback_query_id:
            params = {
                'callback_query_id': callback_query_id,
                'text': self.callback_text,
                'url': self.callback_url,
                'show_alert': self.show_alert,
                'cache_time': self.cache_time,
            }
            response_content = self.tg_bot.answer_callback_query(**params)
            log.info(response_content)



import importlib

import requests
from django.conf import settings

from oscarbot.action import Action
from oscarbot.bot import Bot
from oscarbot.models import Group
from oscarbot.response import TGResponse
from oscarbot.router import Router
from oscarbot.services import get_bot_user_model
from oscarbot.structures import Message


class BaseHandler:

    # logger: Logger

    def __init__(self, token: str, content: dict) -> None:
        self.bot = Bot(token)
        self.content = content
        self.message = Message(content)
        self.user = self.__find_or_create_user_in_db()
        self.group = self.__find_or_create_group_in_db()

    def __find_or_create_group_in_db(self):
        if hasattr(self.message, 'chat'):
            if self.message.chat.type == 'group':
                group_in_db, _ = Group.objects.get_or_create(
                    t_id=self.message.chat.id
                )
                return group_in_db

        return None

    def __find_or_create_user_in_db(self):
        if hasattr(self.message, 'user'):
            first_name = self.message.user.first_name if self.message.user.first_name else ''
            last_name = self.message.user.last_name if self.message.user.last_name else ''
            name = f'{first_name} {last_name}'.strip()
            user_in_db, _ = get_bot_user_model().objects.update_or_create(
                t_id=self.message.user.id,
                defaults={
                    "username": self.message.user.username,
                    "name": name,
                },
            )
            return user_in_db
        return None

    @classmethod
    def __send_do_not_understand(cls):
        need_update, is_delete_message, menu = False, False, None
        message = "Извините, я не понимаю Вас :("
        if getattr(settings, 'NOT_UNDERSTAND_MESSAGE', None):
            message = settings.NOT_UNDERSTAND_MESSAGE
        if getattr(settings, 'NOT_UNDERSTAND_NEED_UPDATE', None):
            need_update = settings.NOT_UNDERSTAND_NEED_UPDATE
        if getattr(settings, 'NOT_UNDERSTAND_IS_DELETE_MESSAGE', None):
            is_delete_message = settings.NOT_UNDERSTAND_IS_DELETE_MESSAGE
        if getattr(settings, 'NOT_UNDERSTAND_MENU', None):
            mod_name, func_name = settings.NOT_UNDERSTAND_MENU.rsplit('.', 1)
            mod = importlib.import_module(mod_name)
            get_menu = getattr(mod, func_name)
            menu = get_menu()
        return TGResponse(message=message, menu=menu, need_update=need_update, is_delete_message=is_delete_message)

    def __work_text_processor(self, photo=None):
        if getattr(settings, 'TELEGRAM_TEXT_PROCESSOR', None):
            response = self.__get_text_handler(photo=photo)
            if response:
                return response
        return self.__send_do_not_understand()

    def handle(self) -> TGResponse:
        if hasattr(self.message, 'data') and self.message.data:
            return self.__handle_callback_data(self.message.data)
        elif hasattr(self.message, 'text') and self.message.text:
            return self.__handle_text_data()
        elif hasattr(self.message, 'photo') and self.message.photo:
            return self.__handle_photo_data()
        elif hasattr(self.message, 'document') and self.message.document:
            return self.__handle_document_data()
        elif hasattr(self.message, 'voice') and self.message.voice:
            return self.__handle_voice_data()
        return self.__work_text_processor()

    def __handle_voice_data(self):
        if getattr(settings, 'TELEGRAM_VOICE_PROCESSOR', None):
            mod_name, func_name = settings.TELEGRAM_VOICE_PROCESSOR.rsplit('.', 1)
            mod = importlib.import_module(mod_name)
            audio_processor = getattr(mod, func_name)
            voice_file = self.bot.get_file(self.message.voice.get('file_id'))
            data = {
                'voice': voice_file,
            }
            response = audio_processor(self.user, data)
            if response:
                return response
        return self.__send_do_not_understand()

    def __get_text_handler(self, photo=None):
        mod_name, func_name = settings.TELEGRAM_TEXT_PROCESSOR.rsplit('.', 1)
        mod = importlib.import_module(mod_name)
        text_processor = getattr(mod, func_name)
        data = {
            'text': self.message.text,
            'photo': photo,
        }
        response = text_processor(self.user, data)
        if response:
            return response

        return False

    def __handle_callback_data(self, path):
        router = Router(path)
        func, arguments = router()
        if func:
            response = func(self.user, **arguments)
            if response:
                self.user.update_path(path)
                return response
        return self.__send_do_not_understand()

    def __handle_text_data(self):
        if self.message.text[0] == '/':
            return self.__handle_callback_data(self.message.text)

        if self.user.want_action:
            action = Action(self.user, self.message.text)
            return action()

        return self.__work_text_processor()

    def __handle_photo_data(self):
        """ WIP: """
        photos = []
        for file in self.message.photo:
            file_id = file['file_id']
            res = requests.get(f'{settings.TELEGRAM_URL}{self.bot.token}/getFile?file_id={file_id}')
            file_path = res.json()['result']['file_path']
            photos.append(
                f'https://api.telegram.org/file/bot{self.bot.token}/{file_path}'
            )
        return self.__work_text_processor(photo=photos)

    def __handle_document_data(self):
        """ WIP: """
        return self.__work_text_processor()



import json
from django.conf import settings
from oscarbot.handler import BaseHandler
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from oscarbot.services import get_bot_model


@csrf_exempt
def bot_view(request, token):
    if request.method == 'POST':
        body = request.body.decode('utf-8')
        body = body.replace('\n', '')
        content = json.loads(body)
        return handle_content(token, content)


def handle_content(token, content):
    if getattr(settings, 'TELEGRAM_API_TOKEN', None):
        bot_token = settings.TELEGRAM_API_TOKEN
    else:
        bot_model = get_bot_model()
        current_bot = bot_model.objects.filter(token=token).first()
        bot_token = current_bot.token if current_bot else None
    if bot_token:
        try:
            handler = BaseHandler(bot_token, content)
            tg_response = handler.handle()
            if tg_response:
                if tg_response.can_send():
                    tg_response.send(token, handler.user, handler.group, content)
                return HttpResponse(content=b"OK")
        except Exception as ex:
            print(ex)
            return HttpResponse(content=b"OK")
    else:
        raise RuntimeError('Failed to find bot')



from django.conf import settings
from django.test import TestCase

from oscarbot.action import Action
from oscarbot.handler import BaseHandler
from oscarbot.menu import Menu
from oscarbot.response import TGResponse
from oscarbot.router import Router
from oscarbot.services import get_bot_user_model


class TGTestCase(TestCase):
    """Base class for testing Telegram bot."""

    def setUp(self):
        """Set up testing environment."""
        self.tg_user = get_bot_user_model().objects.create()

    @staticmethod
    def get_callback_dict(data: str, is_text: bool = True) -> dict:
        """
        Generates content dict for call_text_handler or call_voice_handler.

        :param data: user message text or voice_file_id
        :param is_text: indicates whether the dist is generated for call_text_handler (if True) or call_voice_handler
        :return: Dictionary with arguments for BaseHandler __init__
        """
        content = {
            'callback_query': {
                'message_id': 1,
                'from': {'id': 1, 'is_bot': False, 'first_name': '', 'username': '', 'language_code': 'ru'},
                'chat': {'id': 1, 'first_name': '', 'username': '', 'type': 'private'},
                'date': 1730743912}
        }
        if is_text:
            content['callback_query']['text'] = data
        else:
            content['callback_query']['voice'] = {'file_id': data}
        return content

    def call_router(self, path: str) -> TGResponse:
        """Returns TGResponse based on path."""
        func, args = Router(path)()
        tg_response = func(self.tg_user, **args)
        return tg_response

    def call_action(self, action_path: str) -> TGResponse:
        """Returns TGResponse based on action_path."""
        self.tg_user.want_action = action_path
        tg_response = Action(self.tg_user, '')()
        return tg_response

    def call_text_handler(self, text: str) -> TGResponse:
        """Returns TGResponse based on user input text message."""
        bot_token = None
        if getattr(settings, 'TELEGRAM_API_TOKEN', None):
            bot_token = settings.TELEGRAM_API_TOKEN
        content = self.get_callback_dict(text)
        handler = BaseHandler(bot_token, content)
        tg_response = handler.handle()
        return tg_response

    def call_voice_handler(self, voice_file_id: str) -> TGResponse:
        """Returns TGResponse after procession voice file based on Telegram voice_file_id."""
        bot_token = None
        if getattr(settings, 'TELEGRAM_API_TOKEN', None):
            bot_token = settings.TELEGRAM_API_TOKEN
        content = self.get_callback_dict(voice_file_id, False)
        handler = BaseHandler(bot_token, content)
        tg_response = handler.handle()
        return tg_response

    def assertMenu(self, response: TGResponse, menu: Menu):
        """Asserts that TGResponse contains provided menu."""
        self.assertEqual(len(response.menu.button_list), len(menu.button_list))

        for button_index, response_button in enumerate(response.menu.button_list):
            input_button = menu.button_list[button_index]
            self.assertEqual(response_button.text, input_button.text)
            self.assertEqual(response_button.callback, input_button.callback)
            self.assertEqual(response_button.url, input_button.url)
            self.assertEqual(response_button.web_app, input_button.web_app)

    def assertMessage(self, response: TGResponse, message: str):
        """Asserts that TGResponse 'message' param is equal to provided body."""
        self.assertEqual(response.message, message)

    def assertNeedUpdate(self, response: TGResponse, need_update: bool):
        """Asserts that TGResponse 'need_update' param is equal to provided one."""
        self.assertEqual(response.need_update, need_update)

    def assertUserAction(self, action: str):
        """Asserts that TGUser 'want_action' param is equal to provided one."""
        self.assertEqual(self.tg_user.want_action, action)
